/* ************************************************************************** */
/*                                       _____           _                    */
/*  main.c                              |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : OMGCLOCK                  | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/03/28 12:11:02                               |___/     */
/*                                                                            */
/* ************************************************************************** */
/*	This program is free software: you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation, either version 3 of the License, or
**	(at your option) any later version.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <libfx.h>
#include <librtc.h>
#include <main.h>
#define MODE_VIEW 1
#define MODE_SETTING 2

/*
**	Useful key defines
*/

#define K_EXE 0x21
#define K_EXIT 0x37
#define K_UP 0x18
#define K_LEFT 0x28
#define K_RIGHT 0x17
#define K_DOWN 0x27

/*
**	printdate:
**	Displays the date on screen.
**	@date: the date to display
*/

static void printdate(rtc_date_t date)
{
	static unsigned char s[11] = "00/00/0000";
	static unsigned char t[11] = " 00:00:00";

	// put day
	s[0] = date.day / 10 + '0';
	s[1] = date.day % 10 + '0';
	// put month
	s[3] = date.month / 10 + '0';
	s[4] = date.month % 10 + '0';
	// put year
	s[6] = date.year / 1000 + '0';
	date.year %= 1000;
	s[7] = date.year / 100 + '0';
	date.year %= 100;
	s[8] = date.year / 10 + '0';
	s[9] = date.year % 10 + '0';

	// put hour
	t[1] = date.hour / 10 + '0';
	t[2] = date.hour % 10 + '0';
	// put minute
	t[4] = date.min / 10 + '0';
	t[5] = date.min % 10 + '0';
	// put second
	t[7] = date.sec / 10 + '0';
	t[8] = date.sec % 10 + '0';

	// display on screen
	locate(6, 2);
	Print(s);
	locate(6, 4);
	Print(t);
}

/*
**	changesetting:
**	Changes the setting.
*/

static void changesetting(int oldsetting, int newsetting, int eraseonly)
{
	/* empty/top/bottom four/two */
	static unsigned char ef[5] = "    ";
	static unsigned char et[3] = "  ";
	static unsigned char tf[9] = {0xE6, 0x9C, 0xE6, 0x9C, 0xE6, 0x9C, 0xE6, 0x9C, 0x00};
	static unsigned char tt[5] = {0xE6, 0x9C, 0xE6, 0x9C, 0x00};
	static unsigned char bf[9] = {0xE6, 0x9D, 0xE6, 0x9D, 0xE6, 0x9D, 0xE6, 0x9D, 0x00};
	static unsigned char bt[5] = {0xE6, 0x9D, 0xE6, 0x9D, 0x00};

	/*
	**	Settings are :
	**	- 0 for Days
	**	- 1 for Months
	**	- 2 for Years
	**	- 3 for Hours
	**	- 4 for Minutes
	**	- 5 for Seconds
	*/
	// Erase previous
	if (oldsetting < 2) {
		locate(6 + oldsetting * 3, 1);
		Print(et);
		locate(6 + oldsetting * 3, 3);
		Print(et);
	} else if (oldsetting == 2) {
		locate(12, 1);
		Print(ef);
		locate(12, 3);
		Print(ef);
	} else {
		locate(7 + (oldsetting - 3) * 3, 3);
		Print(et);
		locate(7 + (oldsetting - 3) * 3, 5);
		Print(et);
	}
	// Draw new
	if (!eraseonly) {
		if (newsetting < 2) {
			locate(6 + newsetting * 3, 1);
			Print(tt);
			locate(6 + newsetting * 3, 3);
			Print(bt);
		} else if (newsetting == 2) {
			locate(12, 1);
			Print(tf);
			locate(12, 3);
			Print(bf);
		} else {
			locate(7 + (newsetting - 3) * 3, 3);
			Print(tt);
			locate(7 + (newsetting - 3) * 3, 5);
			Print(bt);
		}
	}
}

/*
**	auxprint:
**	Print info according to mode.
**	@mode: the mode
*/

static void auxprint(int mode)
{
	if (mode == MODE_VIEW) {
		locate(1, 6);
		Print((unsigned char*)"[EXE] Modify");
		locate(1, 7);
		Print((unsigned char*)"[MENU] Exit       ");
	} else {
		locate(1, 6);
		Print((unsigned char*)"[EXE] Save  ");
		locate(1, 7);
		Print((unsigned char*)"[EXIT] Do not save");
	}
}

/*
**	main:
**	Main function of the program.
*/

int main(void)
{
	rtc_date_t date = {0};

	// clean VRAM
	Bdisp_AllClr_VRAM();

	int row, col;
	short unsigned int kc;
	/* Initialize RTC */
	if (!rtc_init()) {
		// put error msg
		locate(1, 1);
		Print((unsigned char*)"Calculator seems to");
		locate(1, 2);
		Print((unsigned char*)"be unsupported.");
		Bdisp_PutDisp_DD();
		while (1) {
			GetKeyW(&col, &row, KEYWAIT_HALTON_TIMEROFF, -1, 0, &kc);
		}
	}

	/* Main loop */
	int mode = MODE_VIEW;
	auxprint(mode);
	int setting = 0;
	while (1) {
		/* Display */
		// print date
		if (mode == MODE_VIEW)
			date = rtc_getdate();
		printdate(date);

		// refresh
		Bdisp_PutDisp_DD();

		/* Reaction to key */
		if (GetKeyW(&col, &row, KEYWAIT_HALTON_TIMERON, 0, 0, &kc) == KEYREP_KEYEVENT) {
			row--; col--; // row and col both have an offset of 1 -- adjust
			kc = (col << 4) | row; //keycode &= 0xFF;
			// Change mode
			if (kc == K_EXE) {
				if (mode == MODE_VIEW) {
					mode = MODE_SETTING;
					auxprint(mode);
					changesetting((setting + 1) % 6, setting, 0);
				} else {
					mode = MODE_VIEW;
					rtc_adjustwday(&date);
					rtc_setdate(date);
					auxprint(mode);
					changesetting(setting, 0, 1);
				}
			}

			// Quit setting mode without setting
			if (kc == K_EXIT && mode == MODE_SETTING) { // EXIT
				mode = MODE_VIEW;
				auxprint(mode);
				changesetting(setting, 0, 1);
			}

			// Change setting (field to set)
			if (mode == MODE_SETTING && (kc == K_LEFT || kc == K_RIGHT)) {
			 	int plus = (kc == K_RIGHT) ? 1 : -1;
				int newsetting = (setting + 6 + plus) % 6;
				changesetting(setting, newsetting, 0);
				setting = newsetting;
			}

			// Modify field
			if (mode == MODE_SETTING && (kc == K_DOWN || kc == K_UP)) {
			 	int plus = (kc == K_UP) ? 1 : -1;
				if (setting == 0) // day
					date.day = (date.day + plus + 31 - 1) % 31 + 1;
				else if (setting == 1) // month
					date.month = (date.month + plus + 12 - 1) % 12 + 1;
				else if (setting == 2) // year
					date.year = (date.year + 10000 + plus) % 10000;
				else if (setting == 3) // hour
					date.hour = (date.hour + 24 + plus) % 24;
				else if (setting == 4) // minutes
					date.min = (date.min + 60 + plus) % 60;
				else if (setting == 5) // seconds
					date.sec = (date.sec + 60 + plus) % 60;
			}
		}
	}
	return (0);
}
